@@ -1,21 +1,65 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 from __future__ import absolute_import, unicode_literals
+from collections import defaultdict
 from django.core.urlresolvers import reverse
 from django.db.models.fields import FieldDoesNotExist
-from django.template import RequestContext, Context, Template
+from django.db import models
+from django.template import Context, Template
 from django.template.loader import render_to_string
-from django.utils.encoding import force_unicode, StrAndUnicode
 from django.utils.datastructures import SortedDict
-from django.utils.text import capfirst
+from django.utils.functional import curry
 from django.utils.html import escape
 from django.utils.safestring import mark_safe, SafeData
 from itertools import ifilter, islice
 import warnings
+import inspect
 from .templatetags.django_tables2 import title
-from .utils import A, AttributeDict, Attrs, OrderBy, OrderByTuple, Sequence
+from .utils import A, AttributeDict, OrderBy, OrderByTuple
 
 
-class Column(object):
+funcs = ifilter(curry(hasattr, inspect), ('getfullargspec', 'getargspec'))
+getargspec = getattr(inspect, next(funcs))
+del funcs
+
+
+class Library(object):
+    """
+    A collection of columns.
+    """
+    def __init__(self):
+        self.columns = []
+
+    def register(self, column):
+        self.columns.append(column)
+        return column
+
+    def column_for_field(self, field):
+        """
+        Return a column object suitable for model field.
+
+        :returns: column object of ``None``
+        """
+        # iterate in reverse order as columns are registered in order
+        # of least to most specialised (i.e. Column is registered
+        # first). This also allows user-registered columns to be
+        # favoured.
+        for candidate in reversed(self.columns):
+            if not hasattr(candidate, "from_field"):
+                continue
+            column = candidate.from_field(field)
+            if column is None:
+                continue
+            return column
+
+
+# The library is a mechanism for announcing what columns are available. Its
+# current use is to allow the table metaclass to ask columns if they're a
+# suitable match for a model field, and if so to return an approach instance.
+library = Library()
+
+
+@library.register
+class Column(object):  # pylint: disable=R0902
     """
     Represents a single column of a table.
 
@@ -55,10 +99,10 @@ class Column(object):
     :type    orderable: :class:`bool`
     :param   orderable: If :const:`False`, this column will not be allowed to
                         influence row ordering/sorting.
-    :type        attrs: :class:`Attrs` object
+    :type        attrs: :class:`dict` object
     :param       attrs: HTML attributes to be added to components in the column
 
-    Supported ``Attrs`` keys are:
+    Supported ``attrs`` keys are:
 
     - *th* -- ``<th>`` element in header
     - *td* -- ``<td>`` element in body
@@ -66,10 +110,11 @@ class Column(object):
     """
     #: Tracks each time a Column instance is created. Used to retain order.
     creation_counter = 0
+    empty_values = (None, '')
 
     def __init__(self, verbose_name=None, accessor=None, default=None,
                  visible=True, orderable=None, attrs=None, order_by=None,
-                 sortable=None):
+                 sortable=None, empty_values=None):
         if not (accessor is None or isinstance(accessor, basestring) or
                 callable(accessor)):
             raise TypeError('accessor must be a string or callable, not %s' %
@@ -87,27 +132,19 @@ class Column(object):
             if orderable is None:
                 orderable = sortable
         self.orderable = orderable
-        attrs = attrs or Attrs()
-        if not isinstance(attrs, Attrs):
-            warnings.warn('attrs must be Attrs object, not %s'
-                          % type(attrs).__name__, DeprecationWarning)
-            attrs = Attrs(attrs)
-        self.attrs = attrs
+        self.attrs = attrs or {}
         # massage order_by into an OrderByTuple or None
         order_by = (order_by, ) if isinstance(order_by, basestring) else order_by
         self.order_by = OrderByTuple(order_by) if order_by is not None else None
+        if empty_values is not None:
+            self.empty_values = empty_values
 
         self.creation_counter = Column.creation_counter
         Column.creation_counter += 1
 
     @property
     def default(self):
-        """
-        The default value for cells in this column.
-
-        The default value passed into ``Column.default`` property may be a
-        callable, this function handles access.
-        """
+        # handle callables
         return self._default() if callable(self._default) else self._default
 
     @property
@@ -142,6 +179,13 @@ class Column(object):
 
         This method can be overridden by :meth:`render_FOO` methods on the
         table or by subclassing :class:`Column`.
+
+        :returns: `unicode`
+
+        If the value for this cell is in `self.empty_values`, this method is
+        skipped and an appropriate default value is rendered instead.
+        Subclasses should set `empty_values` to `()` if they want to handle
+        all values in `render`.
         """
         return value
 
@@ -154,7 +198,68 @@ class Column(object):
                       DeprecationWarning)
         return self.orderable
 
+    @classmethod
+    def from_field(cls, field):
+        """
+        Return a specialised column for the model field or ``None``.
 
+        :param field: the field that needs a suitable column
+        :type  field: model field instance
+        :returns: Column object or ``None``
+
+        If the column isn't specialised for the given model field, it should
+        return ``None``. This gives other columns the opportunity to do better.
+
+        If the column is specialised, it should return an instance of itself
+        that's configured appropriately for the field.
+        """
+        # Since this method is inherited by every subclass, only prove
+        if cls is Column:
+            return cls(verbose_name=field.verbose_name)
+
+
+@library.register
+class BooleanColumn(Column):
+    """
+    A column suitable for rendering boolean data.
+
+    :param  null: is ``None`` different from ``False``?
+    :type   null: bool
+    :param yesno: text to display for True/False values, comma separated
+    :type  yesno: iterable or string
+
+    Rendered values are wrapped in a ``<span>`` to allow customisation by
+    themes. By default the span is given the class ``true``, ``false``.
+
+    In addition to ``attrs`` keys supported by ``Column``, the following are
+    available:
+
+    - *span* -- adds attributes to the <span> tag
+    """
+    def __init__(self, null=False, yesno="✔,✘", **kwargs):
+        self.yesno = (yesno.split(',') if isinstance(yesno, basestring)
+                                       else tuple(yesno))
+        if null:
+            kwargs[b"empty_values"] = ()
+        super(BooleanColumn, self).__init__(**kwargs)
+
+    def render(self, value):
+        value = bool(value)
+        text = self.yesno[int(not value)]
+        html = '<span %s>%s</span>'
+        attrs = {"class": unicode(value).lower()}
+        attrs.update(self.attrs.get("span", {}))
+        return mark_safe(html % (AttributeDict(attrs).as_html(), escape(text)))
+
+    @classmethod
+    def from_field(cls, field):
+        if isinstance(field, models.BooleanField):
+            return cls(verbose_name=field.verbose_name, null=False)
+        if isinstance(field, models.NullBooleanField):
+            return cls(verbose_name=field.verbose_name, null=True)
+
+
+@library.register
 class CheckBoxColumn(Column):
     """
     A subclass of :class:`.Column` that renders as a checkbox form input.
@@ -180,42 +285,45 @@ class CheckBoxColumn(Column):
         implemented. If you want something to actually happen, you'll need to
         implement that yourself.
 
-    In addition to ``Attrs`` keys supported by ``Column``, the following are
+    In addition to ``attrs`` keys supported by ``Column``, the following are
     available:
 
     - *input*     -- ``<input>`` elements in both ``<td>`` and ``<th>``.
-    - *th__input* -- If defined: used *instead of* ``input`` in table header.
-    - *td__input* -- If defined: used *instead of* ``input`` in table body.
+    - *th__input* -- Replaces *input* attrs in header cells.
+    - *td__input* -- Replaces *input* attrs in body cells.
     """
     def __init__(self, attrs=None, **extra):
-        header_attrs = extra.pop('header_attrs', None)
         # For backwards compatibility, passing in a normal dict effectively
         # should assign attributes to the `<input>` tag.
-        attrs = attrs or Attrs()
-        if not isinstance(attrs, Attrs):
-            warnings.warn('attrs must be Attrs object, not %s'
-                          % type(attrs).__name__, DeprecationWarning)
-            attrs = Attrs(td__input=attrs)
+        valid = set(("input", "th__input", "td__input", "th", "td", "cell"))
+        if attrs and not set(attrs) & set(valid):
+            # if none of the keys in attrs are actually valid, assume it's some
+            # old code that should be be interpreted as {"td__input": ...}
+            warnings.warn('attrs keys must be one of %s, interpreting as {"td__input": %s}'
+                          % (', '.join(valid), attrs), DeprecationWarning)
+            attrs = {"td__input": attrs}
         # This is done for backwards compatible too, there used to be a
         # ``header_attrs`` argument, but this has been deprecated. We'll
         # maintain it for a while by translating it into ``head.checkbox``.
-        if header_attrs:
-            attrs.setdefault('th__input', header_attrs)
+        if "header_attrs" in extra:
+            warnings.warn('header_attrs argument is deprecated, '
+                          'use attrs={"th__input": ...} instead',
+                          DeprecationWarning)
+            attrs.setdefault('th__input', {}).update(extra.pop('header_attrs'))
 
         kwargs = {b'orderable': False, b'attrs': attrs}
         kwargs.update(extra)
         super(CheckBoxColumn, self).__init__(**kwargs)
 
-
     @property
     def header(self):
         default = {'type': 'checkbox'}
         general = self.attrs.get('input')
         specific = self.attrs.get('th__input')
         attrs = AttributeDict(default, **(specific or general or {}))
-        return mark_safe(u'<input %s/>' % attrs.as_html())
+        return mark_safe('<input %s/>' % attrs.as_html())
 
-    def render(self, value, bound_column):
+    def render(self, value, bound_column):  # pylint: disable=W0221
         default = {
             'type': 'checkbox',
             'name': bound_column.name,
@@ -224,9 +332,10 @@ class CheckBoxColumn(Column):
         general = self.attrs.get('input')
         specific = self.attrs.get('td__input')
         attrs = AttributeDict(default, **(specific or general or {}))
-        return mark_safe(u'<input %s/>' % attrs.as_html())
+        return mark_safe('<input %s/>' % attrs.as_html())
 
 
+@library.register
 class BaseLinkColumn(Column):
     """
     The base for other columns that render links.
@@ -235,24 +344,35 @@ class BaseLinkColumn(Column):
     ``<a href="...">`` tag.
     """
     def __init__(self, attrs=None, *args, **kwargs):
-        # backwards compatible translation for naive attrs value
-        attrs = attrs or Attrs()
-        if not isinstance(attrs, Attrs):
-            warnings.warn('attrs must be Attrs object, not %s'
-                          % type(attrs).__name__, DeprecationWarning)
-            attrs = Attrs(a=attrs)
+        valid = set(("a", "th", "td", "cell"))
+        if attrs and not set(attrs) & set(valid):
+            # if none of the keys in attrs are actually valid, assume it's some
+            # old code that should be be interpreted as {"a": ...}
+            warnings.warn('attrs keys must be one of %s, interpreting as {"a": %s}'
+                          % (', '.join(valid), attrs), DeprecationWarning)
+            attrs = {"a": attrs}
         kwargs[b'attrs'] = attrs
         super(BaseLinkColumn, self).__init__(*args, **kwargs)
 
     def render_link(self, uri, text, attrs=None):
-        html = u'<a href="{uri}" {attrs}>{text}</a>'.format(
+        """
+        Render a hyperlink.
+
+        :param   uri: URI for the hyperlink
+        :param  text: value wrapped in ``<a></a>``
+        :param attrs: ``<a>`` tag attributes
+        """
+        attrs = AttributeDict(attrs if attrs is not None else
+                              self.attrs.get('a', {}))
+        html = '<a href="{uri}"{attrs}>{text}</a>'.format(
             uri=escape(uri),
-            attrs=attrs or AttributeDict(self.attrs.get('a', {})).as_html(),
+            attrs=" %s" % attrs.as_html() if attrs else "",
             text=escape(text)
         )
         return mark_safe(html)
 
 
+@library.register
 class LinkColumn(BaseLinkColumn):
     """
     Renders a normal value as an internal hyperlink to another page.
@@ -297,7 +417,7 @@ class LinkColumn(BaseLinkColumn):
         class PeopleTable(tables.Table):
             name = tables.LinkColumn('people_detail', args=[A('pk')])
 
-    In addition to ``Attrs`` keys supported by ``Column``, the following are
+    In addition to ``attrs`` keys supported by ``Column``, the following are
     available:
 
     - *a* -- ``<a>`` elements in ``<td>``.
@@ -311,17 +431,7 @@ class LinkColumn(BaseLinkColumn):
         self.kwargs = kwargs
         self.current_app = current_app
 
-    def render(self, value, record, bound_column):
-        # Remember that value is actually what would have normally been put
-        # into the cell. i.e. it *already* takes into consideration the
-        # column's *default* property, thus we must check the actual data value
-        # and use that to decide whether to render a link or just the default
-        try:
-            raw = bound_column.accessor.resolve(record)
-        except (TypeError, AttributeError, KeyError, ValueError):
-            raw = None
-        if raw is None:
-            return self.default
+    def render(self, value, record, bound_column):  # pylint: disable=W0221
         # The following params + if statements create the arguments required to
         # pass to Django's reverse() function.
         params = {}
@@ -338,11 +448,11 @@ class LinkColumn(BaseLinkColumn):
                               for a in self.args]
         if self.kwargs:
             params[b'kwargs'] = {}
-            for k, v in self.kwargs.items():
+            for key, val in self.kwargs.items():
                 # If we're dealing with an Accessor (A), resolve it, otherwise
                 # use the value verbatim.
-                params[b'kwargs'][k] = (v.resolve(record) if isinstance(v, A)
-                                       else v)
+                params[b'kwargs'][str(key)] = (val.resolve(record)
+                                               if isinstance(val, A) else val)
         if self.current_app:
             params[b'current_app'] = (self.current_app.resolve(record)
                                      if isinstance(self.current_app, A)
@@ -350,6 +460,7 @@ class LinkColumn(BaseLinkColumn):
         return self.render_link(reverse(**params), value)
 
 
+@library.register
 class URLColumn(BaseLinkColumn):
     """
     A subclass of :class:`.BaseLinkColumn` that renders the cell value as a hyperlink.
@@ -374,11 +485,16 @@ class URLColumn(BaseLinkColumn):
             web = tables.URLColumn()
 
     """
-
     def render(self, value):
         return self.render_link(value, value)
 
+    @classmethod
+    def from_field(cls, field):
+        if isinstance(field, models.URLField):
+            return cls(verbose_name=field.verbose_name)
 
+
+@library.register
 class EmailColumn(BaseLinkColumn):
     """
     A subclass of :class:`.BaseLinkColumn` that renders the cell value as a hyperlink.
@@ -403,33 +519,41 @@ class EmailColumn(BaseLinkColumn):
             email = tables.EmailColumn()
 
     """
-
     def render(self, value):
         return self.render_link("mailto:%s" % value, value)
 
+    @classmethod
+    def from_field(cls, field):
+        if isinstance(field, models.EmailField):
+            return cls(verbose_name=field.verbose_name)
 
+
+@library.register
 class TemplateColumn(Column):
     """
     A subclass of :class:`.Column` that renders some template code to use as
     the cell value.
 
-    :type template_code: :class:`basestring` object
-    :type template_name: :class:`basestring` object
+    :type  template_code: `unicode`
     :param template_code: the template code to render
+    :type  template_name: `unicode`
     :param template_name: the name of the template to render
 
     A :class:`django.templates.Template` object is created from the
-    *template_code* or *template_name* and rendered with a context containing only a ``record``
-    variable. This variable is the record for the table row being rendered.
+    *template_code* or *template_name* and rendered with a context containing:
+
+    - `record` -- data record for the current row
+    - `value` -- value from `record` that corresponds to the current column
+    - `default` -- appropriate default value to use as fallback
 
     Example:
 
     .. code-block:: python
 
-        class SimpleTable(tables.Table):
-            name1 = tables.TemplateColumn('{{ record.name }}')
-            name2 = tables.TemplateColumn(template_name='myapp/name2_column.html')
-            name3 = tables.Column()
+        class ExampleTable(tables.Table):
+            foo = tables.TemplateColumn('{{ record.bar }}')
+            # contents of `myapp/bar_column.html` is `{{ value }}`
+            bar = tables.TemplateColumn(template_name='myapp/name2_column.html')
 
     Both columns will have the same output.
 
@@ -439,6 +563,8 @@ class TemplateColumn(Column):
         ``RequestContext``, the table **must** be rendered via
         :ref:`{% render_table %} <template-tags.render_table>`.
     """
+    empty_values = ()
+
     def __init__(self, template_code=None, template_name=None, **extra):
         super(TemplateColumn, self).__init__(**extra)
         self.template_code = template_code
@@ -446,12 +572,13 @@ class TemplateColumn(Column):
         if not self.template_code and not self.template_name:
             raise ValueError('A template must be provided')
 
-    def render(self, record, table, **kwargs):
+    def render(self, record, table, value, bound_column, **kwargs):
         # If the table is being rendered using `render_table`, it hackily
         # attaches the context to the table as a gift to `TemplateColumn`. If
         # the table is being rendered via `Table.as_html`, this won't exist.
         context = getattr(table, 'context', Context())
-        context.update({'record': record})
+        context.update({'default': bound_column.default,
+                        'record': record, 'value': value})
         try:
             if self.template_code:
                 return Template(self.template_code).render(context)
@@ -461,6 +588,53 @@ class TemplateColumn(Column):
             context.pop()
 
 
+@library.register
+class DateColumn(TemplateColumn):
+    """
+    A column that renders dates in the local timezone.
+
+    :param format: format string in same format as Django's ``date`` template
+                   filter (optional)
+    :type  format: ``unicode``
+    :param  short: if ``format`` is not specified, use Django's
+                   ``SHORT_DATE_FORMAT`` setting, otherwise use ``DATE_FORMAT``
+    :type   short: ``bool``
+    """
+    def __init__(self, format=None, short=True, *args, **kwargs):
+        if format is None:
+            format = 'SHORT_DATE_FORMAT' if short else 'DATE_FORMAT'
+        template = '{{ value|date:"%s"|default:default }}' % format
+        super(DateColumn, self).__init__(template_code=template, *args, **kwargs)
+
+    @classmethod
+    def from_field(cls, field):
+        if isinstance(field, models.DateField):
+            return cls(verbose_name=field.verbose_name)
+
+
+@library.register
+class DateTimeColumn(TemplateColumn):
+    """
+    A column that renders datetimes in the local timezone.
+
+    :param format: format string for datetime (optional)
+    :type  format: ``unicode``
+    :param  short: if ``format`` is not specifid, use Django's
+                   ``SHORT_DATETIME_FORMAT``, else ``DATETIME_FORMAT``
+    :type   short: ``bool``
+    """
+    def __init__(self, format=None, short=True, *args, **kwargs):
+        if format is None:
+            format = 'SHORT_DATETIME_FORMAT' if short else 'DATETIME_FORMAT'
+        template = '{{ value|date:"%s"|default:default }}' % format
+        super(DateTimeColumn, self).__init__(template_code=template, *args, **kwargs)
+
+    @classmethod
+    def from_field(cls, field):
+        if isinstance(field, models.DateTimeField):
+            return cls(verbose_name=field.verbose_name)
+
+
 class BoundColumn(object):
     """
     A *run-time* version of :class:`.Column`. The difference between
@@ -487,9 +661,9 @@ class BoundColumn(object):
 
     """
     def __init__(self, table, column, name):
-        self._table = table
-        self._column = column
-        self._name = name
+        self.table = table
+        self.column = column
+        self.name = name
 
     def __unicode__(self):
         return unicode(self.header)
@@ -511,7 +685,7 @@ class BoundColumn(object):
         what's actually defined in the column attrs. This makes writing
         templates easier.
         """
-        # Work on a copy of the Attrs object since we're tweaking stuff
+        # Work on a copy of the attrs object since we're tweaking stuff
         attrs = dict(self.column.attrs)
 
         # Find the relevant th attributes (fall back to cell if th isn't
@@ -519,8 +693,8 @@ class BoundColumn(object):
         attrs["td"] = td = AttributeDict(attrs.get('td', attrs.get('cell', {})))
         attrs["th"] = th = AttributeDict(attrs.get("th", attrs.get("cell", {})))
         # make set of existing classes.
-        th_class = set((c for c in th.get("class", "").split(" ") if c))
-        td_class = set((c for c in td.get("class", "").split(" ") if c))
+        th_class = set((c for c in th.get("class", "").split(" ") if c))  # pylint: disable=C0103
+        td_class = set((c for c in td.get("class", "").split(" ") if c))  # pylint: disable=C0103
         # add classes for ordering
         if self.orderable:
             th_class.add("orderable")
@@ -530,23 +704,21 @@ class BoundColumn(object):
         # Always add the column name as a class
         th_class.add(self.name)
         td_class.add(self.name)
-        if th_class: th['class'] = " ".join(sorted(th_class))
-        if td_class: td['class'] = " ".join(sorted(td_class))
+        if th_class:
+            th['class'] = " ".join(sorted(th_class))
+        if td_class:
+            td['class'] = " ".join(sorted(td_class))
         return attrs
 
-    @property
-    def column(self):
-        """
-        Returns the :class:`.Column` object for this column.
-        """
-        return self._column
-
     @property
     def default(self):
         """
         Returns the default value for this column.
         """
-        return self.column.default
+        value = self.column.default
+        if value is None:
+            value = self.table.default
+        return value
 
     @property
     def header(self):
@@ -566,13 +738,6 @@ class BoundColumn(object):
             return verbose_name
         return title(verbose_name)
 
-    @property
-    def name(self):
-        """
-        Returns the string used to identify this column.
-        """
-        return self._name
-
     @property
     def order_by(self):
         """
@@ -584,8 +749,8 @@ class BoundColumn(object):
         if self.column.order_by is not None:
             order_by = self.column.order_by
         else:
-            # default to using column name as data source sort key
-            order_by = OrderByTuple((self.name, ))
+            # default to using column accessor as data source sort key
+            order_by = OrderByTuple((self.accessor, ))
         return order_by.opposite if self.order_by_alias.is_descending else order_by
 
     @property
@@ -601,7 +766,7 @@ class BoundColumn(object):
         the *name* of the column, rather than the keys used to order the table
         data. Understanding the difference is essential.
 
-        Having an alias *and* a normal version is necessary because an N-tuple
+        Having an alias *and* a keys version is necessary because an N-tuple
         (of data source keys) can be used by the column to order the data, and
         it's ambiguous when mapping from N-tuple to column (since multiple
         columns could use the same N-tuple).
@@ -665,13 +830,6 @@ class BoundColumn(object):
             return self.column.orderable
         return self.table.orderable
 
-    @property
-    def table(self):
-        """
-        Return the :class:`Table` object that this column is part of.
-        """
-        return self._table
-
     @property
     def verbose_name(self):
         """
@@ -744,6 +902,16 @@ class BoundColumns(object):
     """
     def __init__(self, table):
         self.table = table
+        self.columns = SortedDict()
+        for name, column in self.table.base_columns.iteritems():
+            self.columns[name] = BoundColumn(self.table, column, name)
+
+        # Prepare each column's ``render`` function and its expected argument
+        # so they can be easily called when each row is iterated.
+        for name, bound_column in self.iteritems():
+            bound_column.render = getattr(self.table, 'render_' + bound_column.name,
+                                          bound_column.column.render)
+            bound_column._render_args = getargspec(bound_column.render).args[1:]
 
     def iternames(self):
         return (name for name, column in self.iteritems())
@@ -770,19 +938,9 @@ class BoundColumns(object):
         consideration all of the ordering and filtering modifiers that a table
         supports (e.g. ``exclude`` and ``sequence``).
         """
-        # First we build a sorted dict of all the columns that we need.
-        columns = SortedDict()
-        for name, column in self.table.base_columns.iteritems():
-            columns[name] = BoundColumn(self.table, column, name)
-
-        # A list of column names in the correct sequence that they should be
-        # rendered in the table.
-        sequence = (self.table.sequence or Sequence(('...', )))
-        sequence.expand(self.table.base_columns.keys())
-
-        for name in sequence:
+        for name in self.table.sequence:
             if name not in self.table.exclude:
-                yield (name, columns[name])
+                yield (name, self.columns[name])
 
     def items(self):
         return list(self.iteritems())
@@ -862,7 +1020,7 @@ class BoundColumns(object):
         """
         if isinstance(index, int):
             try:
-                return next(islice(self.iterall(), index, index+1))
+                return next(islice(self.iterall(), index, index + 1))
             except StopIteration:
                 raise IndexError
         elif isinstance(index, basestring):
@@ -872,5 +1030,12 @@ class BoundColumns(object):
             raise KeyError("Column with name '%s' does not exist; "
                            "choices are: %s" % (index, self.names()))
         else:
-            raise TypeError(u'row indices must be integers or str, not %s'
+            raise TypeError('row indices must be integers or str, not %s'
                             % type(index).__name__)
+
+
+# A dict of ``{model_field: column, ...}``
+model_field_columns = defaultdict(lambda: Column, **{
+    models.DateTimeField: DateTimeColumn,
+    models.DateField: DateColumn,
+    })
