@@ -1,10 +1,6 @@
-# -*- coding: utf-8 -*-
-import inspect
-from itertools import imap, ifilter
+# coding: utf-8
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
-from django.utils.functional import curry
-from django.utils.safestring import EscapeUnicode, SafeData
 from .utils import A
 
 
@@ -62,15 +58,15 @@ class BoundRow(object):
         ...
         KeyError: 'c'
 
-    :param table: is the :class:`Table` in which this row exists.
+    :param  table: is the :class:`Table` in which this row exists.
     :param record: a single record from the :term:`table data` that is used to
-        populate the row. A record could be a :class:`Model` object, a
-        :class:`dict`, or something else.
+                   populate the row. A record could be a :class:`Model` object,
+                   a :class:`dict`, or something else.
 
     """
-    def __init__(self, table, record):
-        self._table = table
+    def __init__(self, record, table):
         self._record = record
+        self._table = table
 
     @property
     def table(self):
@@ -82,7 +78,6 @@ class BoundRow(object):
         """
         The data record from the data source which is used to populate this row
         with data.
-
         """
         return self._record
 
@@ -92,67 +87,59 @@ class BoundRow(object):
 
         Under the hood this method just makes a call to
         :meth:`.BoundRow.__getitem__` for each cell.
-
         """
-        for column in self.table.columns:
+        for column, value in self.items():
             # this uses __getitem__, using the name (rather than the accessor)
             # is correct – it's what __getitem__ expects.
-            yield self[column.name]
+            yield value
 
     def __getitem__(self, name):
         """
         Returns the final rendered value for a cell in the row, given the name
         of a column.
-
         """
         bound_column = self.table.columns[name]
 
-        def value():
-            try:
-                # We need to take special care here to allow get_FOO_display()
-                # methods on a model to be used if available. See issue #30.
-                path, _, remainder = bound_column.accessor.rpartition('.')
-                penultimate = A(path).resolve(self.record)
-                # If the penultimate is a model and the remainder is a field
-                # using choices, use get_FOO_display().
-                if isinstance(penultimate, models.Model):
-                    try:
-                        field = penultimate._meta.get_field(remainder)
-                        display = getattr(penultimate, 'get_%s_display' % remainder, None)
-                        if field.choices and display:
-                            raw = display()
-                            remainder = None
-                    except FieldDoesNotExist:
-                        pass
-                # Fall back to just using the original accessor (we just need
-                # to follow the remainder).
-                if remainder:
-                    raw = A(remainder).resolve(penultimate)
-            except (TypeError, AttributeError, KeyError, ValueError):
-                raw = None
-            return raw if raw is not None else bound_column.default
+        value = None
+        try:
+            # We need to take special care here to allow get_FOO_display()
+            # methods on a model to be used if available. See issue #30.
+            path, _, remainder = bound_column.accessor.rpartition('.')
+            penultimate = A(path).resolve(self.record)
+            # If the penultimate is a model and the remainder is a field
+            # using choices, use get_FOO_display().
+            if isinstance(penultimate, models.Model):
+                try:
+                    field = penultimate._meta.get_field(remainder)
+                    display = getattr(penultimate, 'get_%s_display' % remainder, None)
+                    if field.choices and display:
+                        value = display()
+                        remainder = None
+                except FieldDoesNotExist:
+                    pass
+            # Fall back to just using the original accessor (we just need
+            # to follow the remainder).
+            if remainder:
+                value = A(remainder).resolve(penultimate)
+        except (TypeError, AttributeError, KeyError, ValueError):
+            pass
+
+        if value in bound_column.column.empty_values:
+            return bound_column.default
 
         kwargs = {
-            'value':        value,  # already a function, no need to wrap
+            'value':        lambda: value,
             'record':       lambda: self.record,
             'column':       lambda: bound_column.column,
             'bound_column': lambda: bound_column,
             'bound_row':    lambda: self,
             'table':        lambda: self._table,
         }
-        render_FOO = 'render_' + bound_column.name
-        render = getattr(self.table, render_FOO, bound_column.column.render)
 
-        # just give a list of all available methods
-        funcs = ifilter(curry(hasattr, inspect), ('getfullargspec', 'getargspec'))
-        spec = getattr(inspect, next(funcs))
-        # only provide the arguments that the func is interested in
         kw = {}
-        for name in spec(render).args:
-            if name == 'self':
-                continue
-            kw[name] = kwargs[name]()
-        return render(**kw)
+        for arg_name in bound_column._render_args:
+            kw[arg_name] = kwargs[arg_name]()
+        return bound_column.render(**kw)
 
     def __contains__(self, item):
         """Check by both row object and column name."""
@@ -161,40 +148,41 @@ class BoundRow(object):
         else:
             return item in self
 
+    def items(self):
+        """
+        Returns iterator yielding ``(bound_column, cell)`` pairs.
+
+        ``cell`` is ``row[name]`` -- the rendered unicode value that should be
+        ``rendered within ``<td>``.
+        """
+        for column in self.table.columns:
+            yield (column, self[column.name])
+
 
 class BoundRows(object):
     """
     Container for spawning :class:`.BoundRow` objects.
 
-    The :attr:`.Table.rows` attribute is a :class:`.BoundRows` object.
-    It provides functionality that would not be possible with a simple iterator
-    in the table class.
-
-    :type table: :class:`.Table` object
-    :param table: the table in which the rows exist.
+    :param  data: iterable of records
+    :param table: the table in which the rows exist
 
+    This is used for :attr:`.Table.rows`.
     """
-    def __init__(self, table):
+    def __init__(self, data, table):
+        self.data = data
         self.table = table
 
     def __iter__(self):
-        """Convience method for :meth:`.BoundRows.all`"""
-        for record in self.table.data:
-            yield BoundRow(self.table, record)
+        for record in self.data:
+            yield BoundRow(record, table=self.table)
 
     def __len__(self):
-        """Returns the number of rows in the table."""
-        return len(self.table.data)
-
-    # for compatibility with QuerySetPaginator
-    count = __len__
+        return len(self.data)
 
     def __getitem__(self, key):
-        """Allows normal list slicing syntax to be used."""
-        if isinstance(key, slice):
-            return imap(lambda record: BoundRow(self.table, record),
-                        self.table.data[key])
-        elif isinstance(key, int):
-            return BoundRow(self.table, self.table.data[key])
-        else:
-            raise TypeError('Key must be a slice or integer.')
+        """
+        Slicing returns a new :class:`.BoundRows` instance, indexing returns
+        a single :class:`.BoundRow` instance.
+        """
+        container = BoundRows if isinstance(key, slice) else BoundRow
+        return container(self.data[key], table=self.table)
