@@ -1,10 +1,12 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 from __future__ import absolute_import, unicode_literals
-from django.template import Context
-from django.utils.datastructures import SortedDict
+from django.core.handlers.wsgi import WSGIRequest
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
+from django.test.client import FakePayload
 from itertools import chain
+from StringIO import StringIO
+import warnings
 
 
 class Sequence(list):
@@ -26,21 +28,12 @@ class Sequence(list):
 
         :raises: ``ValueError`` if the sequence is invalid for the columns.
         """
-        # validation
-        if self.count("...") > 1:
+        ellipses = self.count("...")
+        if ellipses > 1:
             raise ValueError("'...' must be used at most once in a sequence.")
-        elif "..." in self:
-            # Check for columns in the sequence that don't exist in *columns*
-            extra = (set(self) - set(("...", ))).difference(columns)
-            if extra:
-                raise ValueError("sequence contains columns that do not exist"
-                                 " in the table. Remove '%s'."
-                                 % "', '".join(extra))
-        else:
-            diff = set(self) ^ set(columns)
-            if diff:
-                raise ValueError("sequence does not match columns. Fix '%s' "
-                                 "or possibly add '...'." % "', '".join(diff))
+        elif ellipses == 0:
+            self.append("...")
+
         # everything looks good, let's expand the "..." item
         columns = columns[:]  # don't modify
         head = []
@@ -51,9 +44,10 @@ class Sequence(list):
                 # now we'll start adding elements to the tail
                 target = tail
                 continue
-            else:
-                target.append(columns.pop(columns.index(name)))
-        self[:] = list(chain(head, columns, tail))
+            target.append(name)
+            if name in columns:
+                columns.pop(columns.index(name))
+        self[:] = chain(head, columns, tail)
 
 
 class OrderBy(str):
@@ -64,7 +58,12 @@ class OrderBy(str):
     @property
     def bare(self):
         """
-        Return the :term:`bare <bare orderby>` form.
+        Return the bare form.
+
+        The *bare form* is the non-prefixed form. Typically the bare form is
+        just the ascending form.
+
+        Example: ``age`` is the bare form of ``-age``
 
         :rtype: :class:`.OrderBy` object
         """
@@ -160,8 +159,8 @@ class OrderByTuple(tuple):
         :returns: :class:`bool`
         """
         name = OrderBy(name).bare
-        for o in self:
-            if o.bare == name:
+        for order_by in self:
+            if order_by.bare == name:
                 return True
         return False
 
@@ -185,9 +184,9 @@ class OrderByTuple(tuple):
         :rtype: :class:`.OrderBy` object
         """
         if isinstance(index, basestring):
-            for ob in self:
-                if ob == index or ob.bare == index:
-                    return ob
+            for order_by in self:
+                if order_by == index or order_by.bare == index:
+                    return order_by
             raise KeyError
         return super(OrderByTuple, self).__getitem__(index)
 
@@ -200,6 +199,7 @@ class OrderByTuple(tuple):
 
         :rtype: function
         """
+        # pylint: disable=C0103
         def _cmp(a, b):
             for accessor, reverse in instructions:
                 x = accessor.resolve(a)
@@ -213,11 +213,11 @@ class OrderByTuple(tuple):
                     return -res if reverse else res
             return 0
         instructions = []
-        for o in self:
-            if o.startswith('-'):
-                instructions.append((Accessor(o[1:]), True))
+        for order_by in self:
+            if order_by.startswith('-'):
+                instructions.append((Accessor(order_by[1:]), True))
             else:
-                instructions.append((Accessor(o), False))
+                instructions.append((Accessor(order_by), False))
         return _cmp
 
     def get(self, key, fallback):
@@ -254,7 +254,7 @@ class Accessor(str):
     """
     SEPARATOR = '.'
 
-    def resolve(self, context):
+    def resolve(self, context, safe=True):
         """
         Return an object described by the accessor by traversing the attributes
         of *context*.
@@ -294,15 +294,18 @@ class Accessor(str):
                 except (TypeError, AttributeError):
                     try:  # list-index lookup
                         current = current[int(bit)]
-                    except (IndexError, # list index out of range
-                            ValueError, # invalid literal for int()
-                            KeyError,   # dict without `int(bit)` key
-                            TypeError,  # unsubscriptable object
+                    except (IndexError,  # list index out of range
+                            ValueError,  # invalid literal for int()
+                            KeyError,    # dict without `int(bit)` key
+                            TypeError,   # unsubscriptable object
                             ):
                         raise ValueError('Failed lookup for key [%s] in %r'
                                          ', when resolving the accessor %s'
                                           % (bit, current, self))
             if callable(current):
+                if safe and getattr(current, 'alters_data', False):
+                    raise ValueError('refusing to call %s() because `.alters_data = True`'
+                                     % repr(current))
                 current = current()
             # important that we break in None case, or a relationship
             # spanning across a null-key will raise an exception in the
@@ -350,16 +353,91 @@ class AttributeDict(dict):
 
 class Attrs(dict):
     """
-    A collection of :class:`AttributeDict`, each given a key.
-
-    This class is used as a container to hold differenct sets of attributes for
-    a given column. Keys indicate where the attributes should be used, and
-    support varies depending on the column.
-
-    It's used in favour of a standard `dict` to enable backwards compatibility.
-    Before it was introduced, columns had an `attrs` parameter that would be
-    given a `dict` and would assign it to a single (typically input) element.
-    The new approach allows attributes to be specified for multiple elements.
-    By using the `Attrs` class your intention to use the new mechanism is
-    explicit.
+    Backwards compatibility, deprecated.
     """
+    def __init__(self, *args, **kwargs):
+        super(Attrs, self).__init__(*args, **kwargs)
+        warnings.warn("Attrs class is deprecated, use dict instead.",
+                      DeprecationWarning)
+
+
+def segment(sequence, aliases):
+    """
+    Translates a flat sequence of items into a set of prefixed aliases.
+
+    This allows the value set by `QuerySet.order_by()` to be translated into
+    a list of columns that would have the same result. These are called
+    "order by aliases" which are optionally prefixed column names.
+
+    e.g.
+
+        >>> list(segment(("a", "-b", "c"),
+        ...              {"x": ("a"),
+        ...               "y": ("b", "-c"),
+        ...               "z": ("-b", "c")}))
+        [["x", "-y"], ["x", "z"]]
+
+    """
+    if not (sequence or aliases):
+        return
+    for alias, parts in aliases.items():
+        variants = {
+            # alias: order by tuple
+            alias:  OrderByTuple(parts),
+            OrderBy(alias).opposite: OrderByTuple(parts).opposite,
+        }
+        for valias, vparts in variants.items():
+            if list(sequence[:len(vparts)]) == list(vparts):
+                tail_aliases = dict(aliases)
+                del tail_aliases[alias]
+                tail_sequence = sequence[len(vparts):]
+                if tail_sequence:
+                    for tail in segment(tail_sequence, tail_aliases):
+                        yield [valias] + tail
+                    else:
+                        continue
+                else:
+                    yield [valias]
+
+
+class cached_property(object):
+    """
+    Decorator that creates converts a method with a single
+    self argument into a property cached on the instance.
+
+    Taken directly from Django 1.4.
+    """
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, instance, type):
+        res = instance.__dict__[self.func.__name__] = self.func(instance)
+        return res
+
+
+def build_request(uri='/'):
+    """
+    Return a fresh HTTP GET / request.
+
+    This is essentially a heavily cutdown version of Django 1.3's
+    ``django.test.client.RequestFactory``.
+    """
+    path, _, querystring = uri.partition('?')
+    return WSGIRequest({
+            'CONTENT_TYPE':      'text/html; charset=utf-8',
+            'PATH_INFO':         path,
+            'QUERY_STRING':      querystring,
+            'REMOTE_ADDR':       '127.0.0.1',
+            'REQUEST_METHOD':    'GET',
+            'SCRIPT_NAME':       '',
+            'SERVER_NAME':       'testserver',
+            'SERVER_PORT':       '80',
+            'SERVER_PROTOCOL':   'HTTP/1.1',
+            'wsgi.version':      (1, 0),
+            'wsgi.url_scheme':   'http',
+            'wsgi.input':        FakePayload(b''),
+            'wsgi.errors':       StringIO(),
+            'wsgi.multiprocess': True,
+            'wsgi.multithread':  False,
+            'wsgi.run_once':     False,
+        })
